'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var redux = require('redux');
var createSelector = require('create-selector');
var createRouteMatcher = _interopDefault(require('feather-route-matcher'));
var reselect = require('reselect');
var idbKeyval = _interopDefault(require('idb-keyval'));
var qs = _interopDefault(require('querystringify'));

var appTimeBundle$1 = {
  name: 'appTime',
  reducer: Date.now,
  selectAppTime: function (state) { return state.appTime; }
};

var changes = {
  'START': 1,
  'SUCCESS': -1,
  'ERROR': -1
};

var re = /_(START|SUCCESS|ERROR)$/;

var asyncCount = {
  name: 'asyncCount',
  reducer: function (state, ref) {
    if ( state === void 0 ) state = 0;
    var type = ref.type;

    var result = re.exec(type);
    if (!result) { return state }
    return state + changes[result[1]]
  },
  selectAsyncActive: function (state) { return state.asyncCount > 0; }
};

var debugMiddleware = function (store) { return function (next) { return function (action) {
  var isDebug = store.getState().debug;

  if (isDebug) {
    console.group(action.type);
    console.info('action:', action);
  }

  var result = next(action);

  if (isDebug) {
    console.debug('state:', store.getState());
    self.logSelectors && self.logSelectors();
    self.logNextReaction && self.logNextReaction();
    console.groupEnd(action.type);
  }

  return result
}; }; };

var namedActionMiddleware = function (store) { return function (next) { return function (action) {
  var actionCreator = action.actionCreator;
  var args = action.args;
  if (actionCreator) {
    var found = store.meta.unboundActionCreators[actionCreator];
    if (!found) {
      throw Error(("NoSuchActionCreator: " + actionCreator))
    }
    return next(args ? found.apply(void 0, args) : found())
  }
  return next(action)
}; }; };

var thunkMiddleware = function (store) { return function (next) { return function (action) {
  if (typeof action === 'function') {
    var getState = store.getState;
    var dispatch = store.dispatch;
    return action(Object.assign({}, {getState: getState, dispatch: dispatch, store: store}, store.meta.extraArgs))
  }
  return next(action)
}; }; };

// Modified to expose all of `store` to middleware instead of just
// `getState` and `dispatch`
var customApplyMiddleware = function () {
  var middlewares = [], len = arguments.length;
  while ( len-- ) middlewares[ len ] = arguments[ len ];

  return function (createStore$$1) { return function (reducer, preloadedState, enhancer) {
  var store = createStore$$1(reducer, preloadedState, enhancer);
  var dispatch = store.dispatch;
  var chain = [];
  chain = middlewares.map(function (middleware) { return middleware(store); });
  dispatch = redux.compose.apply(redux, chain)(store.dispatch);
  return Object.assign(store, { dispatch: dispatch })
}; };
};

var debug = false;
try { debug = !!window.localStorage.debug; } catch (e) {}
var HAS_DEBUG_FLAG = debug;
var HAS_WINDOW = typeof window !== 'undefined';
var IS_BROWSER = HAS_WINDOW || typeof self !== 'undefined';

var fallback = function (func) { setTimeout(func, 0); };
var raf = IS_BROWSER && self.requestAnimationFrame || fallback;
var ric = IS_BROWSER && self.requestIdleCallback || fallback;

var startsWith = function (string, searchString) { return string.substr(0, searchString.length) === searchString; };

var flattenExtractedToObject = function (extracted) {
  var result = {};
  for (var appName in extracted) {
    Object.assign(result, extracted[appName]);
  }
  return result
};

var flattenExtractedToArray = function (extracted) {
  var accum = [];
  for (var appName in extracted) {
    accum.push.apply(accum, extracted[appName]);
  }
  return accum
};

var addGlobalListener = function (eventName, handler) {
  if (IS_BROWSER) {
    self.addEventListener(eventName, handler);
  }
};

var selectorNameToValueName = function (name) {
  var isSelect = startsWith(name, 'select');
  var start = isSelect ? 6 : 5;
  return name.charAt(start).toLowerCase() + name.slice(start + 1)
};

var debounce = function (fn, wait) {
  var timeout;
  var debounced = function () {
    var ctx = this;
    var args = arguments;
    clearTimeout(timeout);
    timeout = setTimeout(function () {
      fn.apply(ctx, args);
    }, wait);
  };
  debounced.cancel = function () {
    clearTimeout(timeout);
  };
  return debounced
};

var normalizeBundle = function (bundle) {
  var name = bundle.name;
  if (!name) { throw TypeError('bundles must have a "name" property') }
  var result = {
    name: name,
    reducer: bundle.reducer || (bundle.getReducer && bundle.getReducer()) || null,
    init: bundle.init || null,
    extraArgs: bundle.extraArgs || null,
    getMiddleware: bundle.getMiddleware,
    actionCreators: null,
    selectors: null,
    reactorNames: null,
    rawBundle: bundle
  };
  Object.keys(bundle).forEach(function (key) {
    if (startsWith(key, 'do')) {
      (result.actionCreators || (result.actionCreators = {}))[key] = bundle[key];
      return
    }
    var isSelector = startsWith(key, 'select');
    var isReactor = startsWith(key, 'react');
    if (isSelector || isReactor) {
      (result.selectors || (result.selectors = {}))[key] = bundle[key];
      if (isReactor) {
        (result.reactorNames || (result.reactorNames = [])).push(key);
      }
      return
    }
  });
  return result
};

var createChunk = function (rawBundles) {
  var normalizedBundles = rawBundles.map(normalizeBundle);
  var result = {
    bundleNames: [],
    reducers: {},
    selectors: {},
    extraArgs: {},
    actionCreators: {},
    rawBundles: [],
    processedBundles: [],
    initMethods: [],
    middlewareCreators: [],
    reactorNames: []
  };
  normalizedBundles.forEach(function (bundle) {
    result.bundleNames.push(bundle.name);
    Object.assign(result.selectors, bundle.selectors);
    Object.assign(result.extraArgs, bundle.extraArgs);
    Object.assign(result.actionCreators, bundle.actionCreators);
    if (bundle.reducer) { Object.assign(result.reducers, ( obj = {}, obj[bundle.name] = bundle.reducer, obj ));
      var obj; }
    if (bundle.init) { result.initMethods.push(bundle.init); }
    if (bundle.middlewareCreators) { result.middlewareCreators.push(bundle.getMiddleware); }
    if (bundle.reactorNames) { (ref = result.reactorNames).push.apply(ref, bundle.reactorNames); }
    result.processedBundles.push(bundle);
    result.rawBundles.push(bundle.rawBundle);
    var ref;
  });
  return result
};

var bindSelectorsToStore = function (store, selectors) {
  var loop = function ( key ) {
    var selector = selectors[key];
    if (!store[key]) {
      store[key] = function () { return selector(store.getState()); };
    }
  };

  for (var key in selectors) loop( key );
};

var decorateStore = function (store, processed) {
  if (!store.meta) {
    store.meta = {
      chunks: [],
      unboundSelectors: {},
      unboundActionCreators: {},
      reactorNames: [],
      extraArgs: {}
    };
  }

  var meta = store.meta;

  // attach for reference
  meta.chunks.push(processed);

  // grab existing unbound (but resolved) selectors, combine with new ones
  var combinedSelectors = Object.assign(meta.unboundSelectors, processed.selectors);

  // run resolver
  createSelector.resolveSelectors(combinedSelectors);

  // update collection of resolved selectors
  meta.unboundSelectors = combinedSelectors;

  // make sure all selectors are bound (won't overwrite if already bound)
  bindSelectorsToStore(store, combinedSelectors);

  // build our list of reactor names
  meta.reactorNames = meta.reactorNames.concat(processed.reactorNames);

  // extend global collections with new stuff
  Object.assign(meta.extraArgs, processed.extraArgs);
  Object.assign(meta.unboundActionCreators, processed.actionCreators);

  // bind and attach only the next action crators to the store
  Object.assign(store, redux.bindActionCreators(processed.actionCreators, store.dispatch));

  // run any new init methods
  processed.initMethods.forEach(function (fn) { return fn(store); });
};

var enableBatchDispatch = function (reducer) { return function (state, action) {
  if (action.type === 'BATCH_ACTIONS') {
    return action.actions.reduce(reducer, state)
  }
  return reducer(state, action)
}; };

var composeBundles$1 = function () {
  var bundles = [], len = arguments.length;
  while ( len-- ) bundles[ len ] = arguments[ len ];

  // build out object of extracted bundle info
  var firstChunk = createChunk(bundles);

  return function (data) {
    // build our list of middleware
    var middleware = [
      namedActionMiddleware,
      thunkMiddleware,
      debugMiddleware ].concat( firstChunk.middlewareCreators.map(function (fn) { return fn(firstChunk); })
    );

    // actually init our store
    var store = redux.createStore(
      enableBatchDispatch(redux.combineReducers(firstChunk.reducers)),
      data,
      customApplyMiddleware.apply(void 0, middleware)
    );

    // upgrade dispatch to take multiple and automatically
    // batch dispatch in that case
    var dispatch = store.dispatch;
    store.dispatch = function () {
        var actions = [], len = arguments.length;
        while ( len-- ) actions[ len ] = arguments[ len ];

        return dispatch(actions.length > 1 ? {type: 'BATCH_ACTIONS', actions: actions} : actions[0]);
    };

    decorateStore(store, firstChunk);

    store.integrateBundles = function () {
      var bundlesToIntegrate = [], len = arguments.length;
      while ( len-- ) bundlesToIntegrate[ len ] = arguments[ len ];

      decorateStore(store, createChunk(bundlesToIntegrate));
      var allReducers = store.meta.chunks.reduce(function (accum, chunk) { return Object.assign(accum, chunk.reducers); }, {});
      store.replaceReducer(enableBatchDispatch(redux.combineReducers(allReducers)));
    };

    return store
  }
};

var createRoutingBundle = function (routes) { return ({
  name: 'routes',
  selectRouteInfo: createSelector.createSelector('selectPathname', createRouteMatcher(routes)),
  selectRouteParams: createSelector.createSelector('selectRouteInfo', function (ref) {
    var params = ref.params;

    return params;
  }),
  selectCurrentComponent: createSelector.createSelector('selectRouteInfo', function (ref) {
    var page = ref.page;

    return page;
  })
}); };

var OFFLINE = 'OFFLINE';
var ONLINE = 'ONLINE';

var onlineBundle$1 = {
  name: 'online',
  selectIsOnline: function (state) { return state.online; },
  reducer: function (state, ref) {
    if ( state === void 0 ) state = true;
    var type = ref.type;

    if (type === OFFLINE) { return false }
    if (type === ONLINE) { return true }
    return state
  },
  init: function (store) {
    addGlobalListener('online', function () { return store.dispatch({type: ONLINE}); });
    addGlobalListener('offline', function () { return store.dispatch({type: OFFLINE}); });
  }
};

var createTimeCheckSelector = function (timeSelector, age, invert) {
  if ( invert === void 0 ) invert = true;

  return reselect.createSelector(
  timeSelector,
  appTimeBundle$1.selectAppTime,
  function (time, appTime) {
    if (!time) {
      return false
    }
    var elapsed = appTime - time;
    if (invert) {
      return elapsed > age
    } else {
      return elapsed < age
    }
  }
);
};

var defaultOpts = {
  name: null,
  getPromise: null,
  actionBaseType: null,
  staleAge: 900000, // fifteen minutes
  retryAfter: 60000, // one minute,
  checkIfOnline: true,
  persist: true
};

var createAsyncResourceBundle$1 = function (spec) {
  var opts = Object.assign({}, defaultOpts, spec);
  var name = opts.name;
  var staleAge = opts.staleAge;
  var retryAfter = opts.retryAfter;
  var actionBaseType = opts.actionBaseType;
  var checkIfOnline = opts.checkIfOnline;
  var ucaseName = name.charAt(0).toUpperCase() + name.slice(1);

  if (process.env.NODE_ENV !== 'production') {
    for (var item in opts) {
      if (opts[item] === null) {
        throw Error(("You must supply an " + item + " option when creating a resource bundle"))
      }
    }
  }

  // build selectors
  var inputSelector = function (state) { return state[name]; };
  var dataSelector = reselect.createSelector(
    inputSelector,
    function (resourceState) { return resourceState.data; }
  );
  var lastSuccessSelector = reselect.createSelector(inputSelector, function (resource) { return resource.lastSuccess; });
  var lastErrorSelector = reselect.createSelector(
    inputSelector,
    function (resource) { return resource.errorTimes.slice(-1)[0] || null; }
  );
  var isStaleSelector = createTimeCheckSelector(lastSuccessSelector, staleAge);
  var isWaitingToRetrySelector = createTimeCheckSelector(lastErrorSelector, retryAfter, false);
  var isLoadingSelector = reselect.createSelector(
    inputSelector,
    function (resourceState) { return resourceState.isLoading; }
  );
  var failedPermanentlySelector = reselect.createSelector(
    inputSelector,
    function (resourceState) { return resourceState.failedPermanantly; }
  );
  var shouldUpdateSelector = reselect.createSelector(
    isLoadingSelector,
    failedPermanentlySelector,
    isWaitingToRetrySelector,
    dataSelector,
    isStaleSelector,
    onlineBundle$1.selectIsOnline,
    function (isLoading, failedPermanantly, isWaitingToRetry, data, isStale, isOnline) {
      if (
        (checkIfOnline && !isOnline) ||
        isLoading ||
        failedPermanantly ||
        isWaitingToRetry
      ) {
        return false
      }
      if (!data) {
        return true
      }
      return isStale
    }
  );

  var actions = {
    START: (actionBaseType + "_START"),
    SUCCESS: (actionBaseType + "_SUCCESS"),
    ERROR: (actionBaseType + "_ERROR"),
    MAKE_STALE: (actionBaseType + "_MAKE_STALE")
  };

  var initialState = {
    isLoading: false,
    data: null,
    errorTimes: [],
    lastSuccess: null,
    stale: false,
    failedPermanantly: false
  };

  var doFetchError = function (error) { return ({type: actions.ERROR, error: error}); };
  var doMarkAsStale = function (error) { return ({type: actions.ERROR, error: error}); };
  var doFetchSuccess = function (payload) { return ({type: actions.SUCCESS, payload: payload}); };
  var doFetchData = function () { return function (args) {
    var dispatch = args.dispatch;
    dispatch({type: actions.START});
    return opts.getPromise(args)
      .then(
        function (payload) { dispatch(doFetchSuccess(payload)); },
        function (error) { dispatch(doFetchError(error)); }
      )
  }; };

  var result = {
    name: name,
    reducer: function (state, ref) {
      if ( state === void 0 ) state = initialState;
      var type = ref.type;
      var payload = ref.payload;
      var error = ref.error;
      var merge = ref.merge;

      if (type === actions.START) {
        return Object.assign({}, state, { isLoading: true })
      }
      if (type === actions.SUCCESS) {
        var updatedData;
        if (merge) {
          updatedData = Object.assign({}, state.data, payload);
        } else {
          updatedData = payload;
        }
        return Object.assign({}, state, {
          isLoading: false,
          data: updatedData,
          lastSuccess: Date.now(),
          errorTimes: [],
          failedPermanantly: false,
          stale: false
        })
      }
      if (type === actions.ERROR) {
        return Object.assign({}, state, {
          isLoading: false,
          errorTimes: state.errorTimes.concat([Date.now()]),
          failedPermanantly: !!(error && error.permanent)
        })
      }
      if (type === actions.MAKE_STALE) {
        return Object.assign({}, state, {
          errorTimes: [],
          stale: true
        })
      }
      return state
    }
  };
  result[("select" + ucaseName + "Raw")] = inputSelector;
  result[("select" + ucaseName)] = dataSelector;
  result[("select" + ucaseName + "IsStale")] = isStaleSelector;
  result[("select" + ucaseName + "LastError")] = lastErrorSelector;
  result[("select" + ucaseName + "IsWaitingToRetry")] = isWaitingToRetrySelector;
  result[("select" + ucaseName + "IsLoading")] = isLoadingSelector;
  result[("select" + ucaseName + "FailedPermanantly")] = failedPermanentlySelector;
  result[("select" + ucaseName + "ShouldUpdate")] = shouldUpdateSelector;
  result[("doFetch" + ucaseName)] = doFetchData;
  result[("doFetch" + ucaseName + "Success")] = doFetchSuccess;
  result[("doFetch" + ucaseName + "Error")] = doFetchError;
  result[("doMark" + ucaseName + "AsStale")] = doMarkAsStale;

  if (opts.persist) {
    result.persistActions = [ actions.SUCCESS ];
  }

  return result
};

// Thin layer on top of idbkey val with support for versioning and
// max age
var defaultOpts$1 = {maxAge: Infinity, version: 0, lib: idbKeyval};

var getCachedItem = function (key, opts) {
  var ref = Object.assign(opts, defaultOpts$1);
  var maxAge = ref.maxAge;
  var version = ref.version;
  var lib = ref.lib;
  return lib.get(key)
    .then(JSON.parse)
    .then(function (parsed) {
      var age = Date.now() - parsed.time;
      if (age > maxAge || version !== parsed.version) {
        lib.delete(key);
        return null
      }
      return {
        age: age,
        data: parsed.data
      }
    })
    .catch(function () { return null; })
};

var getAllCached = function (opts) {
  Object.assign(opts, defaultOpts$1);
  var keys;
  return opts.lib.keys()
    .then(function (retrivedKeys) {
      keys = retrivedKeys;
      return Promise.all(keys.map(function (key) { return getCachedItem(key, opts)
          .then(function (res) { return res.data; }); }
      ))
    })
    .then(function (data) { return data.reduce(function (acc, bundleData, index) {
      if (bundleData) {
        acc[keys[index]] = bundleData;
      }
      return acc
    }, {}); })
    .catch(function () {})
};

var clearAllCached = function (opts) {
    if ( opts === void 0 ) opts = defaultOpts$1;

    return opts.lib.clear().catch(function () { return null; });
};

var cacheItem = function (key, data, opts) {
  Object.assign(opts, defaultOpts$1);
  return opts.lib.set(key, JSON.stringify({
    version: opts.version,
    time: Date.now(),
    data: data
  }))
  .catch(function () { return null; })
};

var defaults = { version: 0, cacheFunction: cacheItem };
var caching = function (spec) {
  var opts = Object.assign({}, defaults, spec);

  return {
    name: 'localCache',
    extract: 'persistActions',
    getMiddleware: function (chunk) {
      var combinedActions = {};
      chunk.rawBundles.forEach(function (bundle) {
        if (bundle.persistActions) {
          bundle.persistActions.forEach(function (type) {
            combinedActions[type] || (combinedActions[type] = []);
            combinedActions[type].push(bundle.name);
          });
        }
      });

      return function (ref) {
        var getState = ref.getState;

        return function (next) { return function (action) {
        var keys = combinedActions[action.type];
        var res = next(action);
        var state = getState();
        if (keys) {
          if (IS_BROWSER) {
            ric(function () {
              Promise.all(keys.map(function (key) { return opts.cacheFunction(key, state[key], {version: opts.version}); }
              )).then(function () {
                if (state.debug) {
                  console.info(("persisted " + (keys.join(', ')) + " because of action " + (action.type)));
                }
              });
            });
          }
        }
        return res
      }; };
      }
    }
  }
};

var getError = function (message, permanent) {
  if ( permanent === void 0 ) permanent = false;

  var err = new Error(message);
  if (permanent) { err.permanent = true; }
  return err
};

var geoErrorArray = [
  'An unknown geolocation error occured',
  'Geolocation permission denied',
  'Geolocation unavailable',
  'Geolocation request timed out'
];

var defaultOpts$2 = {
  timeout: 60000,
  enableHighAccuracy: false,
  persist: true,
  staleAge: 900000, // fifteen minutes
  retryAfter: 60000 // one minute,
};

var geolocation = function (spec) {
  var opts = Object.assign({}, defaultOpts$2, spec);
  return createAsyncResourceBundle$1({
    name: 'geolocation',
    actionBaseType: 'REQUEST_GEOLOCATION',
    getPromise: function () { return new Promise(function (resolve, reject) {
      if (!IS_BROWSER || !navigator.geolocation) {
        reject(getError('Geolocation not supported', true));
      }
      var success = function (position) {
        var res = {};
        var coords = position.coords;
        for (var key in coords) {
          res[key] = coords[key];
        }
        res.timestamp = position.timestamp;
        resolve(res);
      };
      var fail = function (ref) {
        var code = ref.code;

        reject(getError(geoErrorArray[code], code === 1));
      };
      var geoOpts = {
        timeout: opts.timeout,
        enableHighAccuracy: opts.enableHighAccuracy
      };
      navigator.geolocation.getCurrentPosition(success, fail, geoOpts);
    }); },
    persist: opts.persist,
    staleAge: opts.staleAge,
    retryAfter: opts.retryAfter
  })
};

var defaults$1 = {
  idleTimeout: 30000,
  idleAction: 'APP_IDLE',
  doneCallback: null,
  stopWhenTabInactive: true
};

var ricOptions = {timeout: 500};
var getIdleDispatcher$1 = function (stopWhenInactive, timeout, fn) { return debounce(function () {
  // the requestAnimationFrame ensures it doesn't run when tab isn't active
  stopWhenInactive ? raf(function () { return ric(fn, ricOptions); }) : ric(fn, ricOptions);
}, timeout); };

var reactors = function (opts) { return ({
  name: 'reactors',
  init: function (store) {
    opts || (opts = {});
    Object.assign(opts, defaults$1);
    var idleAction = opts.idleAction;
    var idleTimeout = opts.idleTimeout;
    var idleDispatcher;
    if (idleTimeout) {
      idleDispatcher = getIdleDispatcher$1(opts.stopWhenTabInactive, idleTimeout, function () { return store.dispatch({type: idleAction}); });
    }

    if (process.env.NODE_ENV !== 'production') {
      store.meta.reactorNames.forEach(function (name) {
        if (!store[name]) {
          throw Error(("Reactor '" + name + "' not found on the store. Make sure you're defining as selector by that name."))
        }
      });
    }

    var cancelIfDone = function () {
      if (!IS_BROWSER && !store.nextReaction && (!store.selectAsyncActive || !store.selectAsyncActive())) {
        idleDispatcher && idleDispatcher.cancel();
        opts.doneCallback && opts.doneCallback();
      }
    };

    var dispatchNext = function () {
      // one at a time
      if (store.nextReaction) {
        return
      }
      // look for the next one
      store.meta.reactorNames.some(function (name) {
        var result = store[name]();
        if (result) {
          store.activeReactor = name;
          store.nextReaction = result;
        }
        return result
      });
      if (store.nextReaction) {
        // let browser chill
        ric(function () {
          var nextReaction = store.nextReaction;
          store.activeReactor = null;
          store.nextReaction = null;
          store.dispatch(nextReaction);
        }, ricOptions);
      }
    };

    var callback = function () {
      dispatchNext();
      if (idleDispatcher) {
        idleDispatcher();
        cancelIfDone();
      }
    };

    store.subscribe(callback);
    callback();
  }
}); };

var isDefined = function (thing) { return typeof thing !== 'undefined'; };
var ensureString = function (input) { return typeof input === 'string' ? input : qs.stringify(input); };
var IPRE = /^[0-9\.]+$/;
var parseSubdomains = function (hostname, getBareHost) {
  if (IPRE.test(hostname)) { return [] }
  var parts = hostname.split('.');
  if (getBareHost) {
    return parts.slice(-2).join('.')
  }
  return hostname.split('.').slice(0, -2)
};
var removeLeading = function (char, string) { return string.charAt(0) === char ? string.slice(1) : string; };
var ensureLeading = function (char, string) {
  if (string === char || string === '') {
    return ''
  }
  return string.charAt(0) !== char ? char + string : string
};
var loc = (function () {
  if (!HAS_WINDOW) { return {} }
  return window.location
})();
var defaults$2 = {
  name: 'url',
  inert: !HAS_WINDOW,
  actionType: 'UPDATE_URL'
};

var url = function (opts) {
  var config = Object.assign({}, defaults$2, opts);
  var actionType = config.actionType;

  var selectUrlRaw = function (state) { return state[config.name]; };
  var selectUrlObject = createSelector.createSelector(selectUrlRaw, function (urlState) { return new URL(urlState.url); });
  var selectQueryObject = createSelector.createSelector(selectUrlObject, function (urlObj) { return qs.parse(urlObj.search); });
  var selectQueryString = createSelector.createSelector(selectQueryObject, function (queryObj) { return qs.stringify(queryObj); });
  var selectPathname = createSelector.createSelector(selectUrlObject, function (urlObj) { return urlObj.pathname; });
  var selectHash = createSelector.createSelector(selectUrlObject, function (urlObj) { return removeLeading('#', urlObj.hash); });
  var selectHashObject = createSelector.createSelector(selectHash, function (hash) { return qs.parse(hash); });
  var selectHostname = createSelector.createSelector(selectUrlObject, function (urlObj) { return urlObj.hostname; });
  var selectSubdomains = createSelector.createSelector(selectHostname, function (hostname) { return parseSubdomains(hostname); });

  var doUpdateUrl = function (newState, opts) {
    if ( opts === void 0 ) opts = {replace: false};

    return function (ref) {
    var dispatch = ref.dispatch;
    var getState = ref.getState;

    var state = newState;
    if (typeof newState === 'string') {
      var parsed = new URL(newState.charAt(0) === '/' ? 'http://example.com' + newState : newState);
      state = {
        pathname: parsed.pathname,
        query: parsed.search || '',
        hash: parsed.hash || ''
      };
    }
    var url = new URL(selectUrlRaw(getState()).url);
    if (isDefined(state.pathname)) { url.pathname = state.pathname; }
    if (isDefined(state.hash)) { url.hash = ensureString(state.hash); }
    if (isDefined(state.query)) { url.search = ensureString(state.query); }
    dispatch({ type: actionType, payload: { url: url.href, replace: opts.replace } });
  };
  };
  var doReplaceUrl = function (url) { return doUpdateUrl(url, {replace: true}); };
  var doUpdateQuery = function (query, opts) {
      if ( opts === void 0 ) opts = {replace: true};

      return doUpdateUrl({ query: ensureString(query) }, opts);
  };
  var doUpdateHash = function (hash, opts) {
      if ( opts === void 0 ) opts = {replace: false};

      return doUpdateUrl({ hash: ensureLeading('#', ensureString(hash)) }, opts);
  };

  return {
    name: config.name,
    init: function (store) {
      if (config.inert) {
        return
      }

      var lastState = store.selectUrlRaw();

      var setCurrentUrl = function () {
        store.doUpdateUrl({
          pathname: loc.pathname,
          hash: loc.hash,
          query: loc.search
        });
      };

      window.addEventListener('popstate', setCurrentUrl);

      store.subscribe(function () {
        var newState = store.selectUrlRaw();
        var newUrl = newState.url;
        if (lastState !== newState && newUrl !== loc.href) {
          try {
            window.history[newState.replace ? 'replaceState' : 'pushState']({}, null, newState.url);
            document.body.scrollTop = 0;
          } catch (e) {
            console.error(e);
          }
        }
        lastState = newState;
      });
    },
    getReducer: function () {
      var initialState = {
        url: !config.inert && HAS_WINDOW
          ? loc.href
          : '/',
        replace: false
      };

      return function (state, ref) {
        if ( state === void 0 ) state = initialState;
        var type = ref.type;
        var payload = ref.payload;

        if (type === '@@redux/INIT' && typeof state === 'string') {
          return {
            url: state,
            replace: false
          }
        }
        if (type === actionType) {
          return Object.assign({
            url: payload.url || payload,
            replace: !!payload.replace
          })
        }
        return state
      }
    },
    doUpdateUrl: doUpdateUrl,
    doReplaceUrl: doReplaceUrl,
    doUpdateQuery: doUpdateQuery,
    doUpdateHash: doUpdateHash,
    selectUrlRaw: selectUrlRaw,
    selectUrlObject: selectUrlObject,
    selectQueryObject: selectQueryObject,
    selectQueryString: selectQueryString,
    selectPathname: selectPathname,
    selectHash: selectHash,
    selectHashObject: selectHashObject,
    selectHostname: selectHostname,
    selectSubdomains: selectSubdomains
  }
};

var version = "13.0.5";

var ENABLE = 'ENABLE_DEBUG';
var DISABLE = 'DISABLE_DEBUG';

var debug$1 = {
  name: 'debug',
  reducer: function (state, ref) {
    if ( state === void 0 ) state = HAS_DEBUG_FLAG;
    var type = ref.type;

    if (type === ENABLE) {
      return true
    }
    if (type === DISABLE) {
      return false
    }
    return state
  },
  doEnableDebug: function () { return ({ type: ENABLE }); },
  doDisableDebug: function () { return ({ type: DISABLE }); },
  selectIsDebug: function (state) { return state.debug; },
  init: function (store) {
    if (store.selectIsDebug()) {
      var names = store.meta.chunks[0].bundleNames;
      self.store = store;
      var actionCreators = [];
      for (var key in store) {
        if (key.indexOf('do') === 0) {
          actionCreators.push(key);
        }
      }
      actionCreators.sort();
      var colorTitle = 'color: #1676D2;';
      var black = 'color: black;';
      var colorGreen = 'color: #4CAF50;';
      var colorOrange = 'color: #F57C00;';

      store.logSelectors = self.logSelectors = function () {
        if (!store.selectAll) { return }
        console.log('%cselectors:', colorGreen, store.selectAll());
      };

      store.logBundles = self.logBundles = function () {
        console.log('%cinstalled bundles:\n  %c%s', colorTitle, black, names.join('\n  '));
      };

      store.logActionCreators = self.logActionCreators = function () {
        console.groupCollapsed('%caction creators', colorOrange);
        actionCreators.forEach(function (name$$1) { return console.log(name$$1); });
        console.groupEnd();
      };

      store.logReactors = self.logReactors = function () {
        console.groupCollapsed('%creactors', colorOrange);
        var ref = store.meta;
        var reactorNames = ref.reactorNames;
        reactorNames.forEach(function (name$$1) { return console.log(name$$1); });
        console.groupEnd();
      };

      store.logNextReaction = self.logNextReaction = function () {
        var nextReaction = store.nextReaction;
        var activeReactor = store.activeReactor;
        if (nextReaction) {
          console.log(
            ("%cnext reaction:\n  %c" + activeReactor),
            colorOrange,
            black,
            nextReaction
          );
        }
      };

      console.groupCollapsed('%credux bundler v%s', colorTitle, version);
      store.logBundles();
      store.logSelectors();
      store.logReactors();
      console.groupEnd();
      if (store.isReacting) {
        console.log("%cqueuing reaction:", colorOrange);
      }
    }
  }
};

var selectAll = {
  name: 'selectAll',
  init: function (store) {
    store.selectAll = function () {
      var selectorNames = Object.keys(store.meta.unboundSelectors).slice().sort().reverse();
      return selectorNames.reduce(function (accum, key) {
        accum[selectorNameToValueName(key)] = store[key]();
        return accum
      }, {})
    };
  }
};

var appTimeBundle = appTimeBundle$1;
var asyncCountBundle = asyncCount;
var cachingBundle = caching;
var createRouteBundle = createRoutingBundle;
var createAsyncResourceBundle = createAsyncResourceBundle$1;
var reactorsBundle = reactors;
var getIdleDispatcher$$1 = getIdleDispatcher$1;
var onlineBundle = onlineBundle$1;
var urlBundle = url;
var debugBundle = debug$1;
var composeBundlesRaw = composeBundles$1;
var geolocationBundle = geolocation;
var selectAllBundle = selectAll;
var composeBundles = function () {
  var userBundles = [], len = arguments.length;
  while ( len-- ) userBundles[ len ] = arguments[ len ];

  userBundles || (userBundles = []);
  var bundles = [
    appTimeBundle$1,
    asyncCount,
    onlineBundle$1,
    url(),
    reactors(),
    selectAll,
    debug$1 ].concat( userBundles
  );
  return composeBundles$1.apply(void 0, bundles)
};

exports.appTimeBundle = appTimeBundle;
exports.asyncCountBundle = asyncCountBundle;
exports.cachingBundle = cachingBundle;
exports.createRouteBundle = createRouteBundle;
exports.createAsyncResourceBundle = createAsyncResourceBundle;
exports.reactorsBundle = reactorsBundle;
exports.getIdleDispatcher = getIdleDispatcher$$1;
exports.onlineBundle = onlineBundle;
exports.urlBundle = urlBundle;
exports.debugBundle = debugBundle;
exports.composeBundlesRaw = composeBundlesRaw;
exports.geolocationBundle = geolocationBundle;
exports.selectAllBundle = selectAllBundle;
exports.composeBundles = composeBundles;
exports.createSelector = createSelector.createSelector;
exports.resolveSelectors = createSelector.resolveSelectors;
exports.HAS_DEBUG_FLAG = HAS_DEBUG_FLAG;
exports.HAS_WINDOW = HAS_WINDOW;
exports.IS_BROWSER = IS_BROWSER;
exports.raf = raf;
exports.ric = ric;
exports.startsWith = startsWith;
exports.flattenExtractedToObject = flattenExtractedToObject;
exports.flattenExtractedToArray = flattenExtractedToArray;
exports.addGlobalListener = addGlobalListener;
exports.selectorNameToValueName = selectorNameToValueName;
exports.debounce = debounce;
exports.getCachedItem = getCachedItem;
exports.getAllCached = getAllCached;
exports.clearAllCached = clearAllCached;
exports.cacheItem = cacheItem;

